==> locus-0.84.tar.gz: locus-0.84/lib/options/typedopt.h <==
#if !defined(typedopt_h)
#define typedopt_h

#if defined(EXTERNAL_TEMPLATES)
#pragma interface
#endif

#include "optc.h"
#include "mape.h"

#include "composed.h"
#include "slv.h"

template<class CValue>
class COptions : public COptionsCode
{
public:
    typedef CValue (*FCompute)(const COptions<CValue> *, const char *);

private:
    DMap<DString, CValue> ValueMap;

public:
    COptions();
    virtual ~COptions();

    void Add(const char *Key, const CValue &Value);
        // asserts key is not duplicate
    void Add(const char *Key, FCompute Compute);
	// asserts key is not duplicate

    CValue Get(const char *Key) const;
	// asserts key exists;

    // does not throw
    // returns 0 OK, 1 key is not in ValueMap
    short Override(const char *Section,
        const char *Key, 
	const CValue &Value);

    virtual DString GetKey(const char *Section,	const char *Key) const = 0;

    // belongs into CUnoOptions<> (ancestor of COptions<>)
    void Add(const char *Section, const char *Key, const CValue &Value);
    void Add(const char *Section, const char *Key, FCompute Compute);
    void SetUnoClass(const CStringListValue &List,
        const char *Attr[],
        int AttrCount);
    CValue GetUno(const DString &Name, const char *Key) const;
    CValue GetMostSpecific(const DString &Section, const char *Key) const;

private:
    void AssertCanAdd(const DString &Key);
};

template<class CValue>
COptions<CValue>::COptions()
{
}

template<class CValue>
COptions<CValue>::~COptions()
{
}

template<class CValue>
void COptions<CValue>::Add(const char *Key, const CValue &Value)
{
	DString KeyStr = Key;
	AssertCanAdd(KeyStr);
	ValueMap.Add(KeyStr, Value);
}

template<class CValue>
void COptions<CValue>::Add(const char *Key,
	 CValue (*Compute)(const COptions<CValue> *, const char *)) // gcc does not handle typedef
{
	DString KeyStr = Key;
	AssertCanAdd(KeyStr);
	FncMap.Add(KeyStr, reinterpret_cast<FGenCompute>(Compute));
}

template<class CValue>
CValue COptions<CValue>::Get(const char *Key) const
{
	DString KeyStr = Key;

	const CValue *Value = ValueMap.Find(KeyStr);
	if (Value != NULL)
		return *Value;

	const FGenCompute *GenCompute = FncMap.Find(KeyStr);
	if (!GenCompute)
	    // special exception might be cleaner, but this is internal error, so
	    // it's not that important to localize the text (and exceptions don't
            // work anyway)
	{   CMemPtr<char> Err(100 + KeyStr.GetLength());
	    sprintf((char *)Err, "undefined key %s", Key);
	    throw CGeneralException((char *)Err);
        }

	const FCompute Compute = reinterpret_cast<FCompute>(*GenCompute);
	return (*Compute)(this, Key);
}

template<class CValue>
short COptions<CValue>::Override(const char *Section,
	const char *Key, 
	const CValue &Value)
{
	DString KeyStr = GetKey(Section, Key);

	const CValue *OldValuePtr = ValueMap.Find(KeyStr);
	if (!OldValuePtr)
		return 1;

	ValueMap.Del(KeyStr);
	ValueMap.Add(KeyStr, Value);
	return 0;
}

template<class CValue>
CValue COptions<CValue>::GetMostSpecific(const DString &Section, 
    const char *Key) const
{
    if (Section.IsEmpty())
	return Get(Key);

    DString Compo = GetLongOptionKey(Section, Key);
    if (ValueMap.Find(Compo) != NULL || FncMap.Find(Compo) != NULL)
	return Get(Compo.c_str());
    else
	return Get(Key);
}

template<class CValue>
CValue COptions<CValue>::GetUno(const DString &Name, const char *Key) const
{
     if (Name.IsEmpty())
	     return Get(Key);

	DString Compo = GetLongOptionKey(Name, Key);
	return Get(Compo.c_str());
}

template<class CValue>
void COptions<CValue>::Add(const char *Section, 
    const char *Key, 
    const CValue &Value)
{
    if (!Section)
	Add(Key, Value);
    else
    {   DString LongKey = GetLongOptionKey(DString(Section), Key);
	Add(LongKey.c_str(), Get(Key));
    }
}

template<class CValue>
void COptions<CValue>::Add(const char *Section, const char *Key,
    CValue (*Compute)(const COptions<CValue> *, const char *))
{
    if (!Section)
	Add(Key, Compute);
    else
    {   DString LongKey = GetLongOptionKey(DString(Section), Key);  
	Add(LongKey.c_str(), Get(Key));
    }
}

template<class CValue>
void COptions<CValue>::SetUnoClass(const CStringListValue &List,
     const char *Attr[],
     int AttrCount)
{
    CStringListValue::DIterator Itr = const_cast<CStringListValue &>(
            List).InitIterator();
    while (const_cast<CStringListValue &>(List).IsDereferenceable(Itr))
    {   for (int i = 0; i < AttrCount; i++)
	{   DString Key = GetLongOptionKey(*Itr, Attr[i]);   
	    Add(Key.c_str(), Get(Attr[i]));
	}
	++Itr;
    }
}

template<class CValue>
void COptions<CValue>::AssertCanAdd(const DString &Key)
{
	const CValue *ValuePtr = ValueMap.Find(Key);
	Assert(!ValuePtr);

	const FGenCompute *GenCompute = FncMap.Find(Key);
	Assert(!GenCompute);
}

#endif

==> locus-0.88.tar.gz: locus-0.88/lib/options/typedopt.h <==
#if !defined(typedopt_h)
#define typedopt_h

#if defined(EXTERNAL_TEMPLATES)
#pragma interface
#endif

#include "optc.h"
#include "mape.h"

#include "composed.h"
#include "slv.h"

template<class CValue>
class COptions : public COptionsCode
{
public:
    typedef CValue (*FCompute)(const COptions<CValue> *, const char *);

private:
    DMap<DString, CValue> ValueMap;

public:
    COptions();
    virtual ~COptions();

    void Add(const char *Key, const CValue &Value);
        // asserts key is not duplicate
    void Add(const char *Key, FCompute Compute);
	// asserts key is not duplicate

    CValue Get(const char *Key) const;
	// asserts key exists;

    // does not throw
    // returns 0 OK, 1 key is not in ValueMap
    short Override(const char *Section,
        const char *Key, 
	const CValue &Value);

    virtual DString GetKey(const char *Section,	const char *Key) const = 0;

    // belongs into CUnoOptions<> (ancestor of COptions<>)
    void Add(const char *Section, const char *Key, const CValue &Value);
    void Add(const char *Section, const char *Key, FCompute Compute);
    void SetUnoClass(const CStringListValue &List,
        const char *Attr[],
        int AttrCount);
    CValue GetUno(const DString &Name, const char *Key) const;
    CValue GetMostSpecific(const DString &Section, const char *Key) const;

    inline void AssertCanAdd(const DString &Key)
    {
#if !defined(NO_ASSERT)
	const CValue *ValuePtr = ValueMap.Find(Key);
	Assert(!ValuePtr);

	const FGenCompute *GenCompute = FncMap.Find(Key);
	Assert(!GenCompute);
#endif
    }
};

template<class CValue>
COptions<CValue>::COptions()
{
}

template<class CValue>
COptions<CValue>::~COptions()
{
}

template<class CValue>
void COptions<CValue>::Add(const char *Key, const CValue &Value)
{
	DString KeyStr = Key;
	AssertCanAdd(KeyStr);
	ValueMap.Add(KeyStr, Value);
}

template<class CValue>
void COptions<CValue>::Add(const char *Key,
	 CValue (*Compute)(const COptions<CValue> *, const char *))
{
	DString KeyStr = Key;
	AssertCanAdd(KeyStr);
	FncMap.Add(KeyStr, reinterpret_cast<FGenCompute>(Compute));
}

template<class CValue>
CValue COptions<CValue>::Get(const char *Key) const
{
	DString KeyStr = Key;

	const CValue *Value = ValueMap.Find(KeyStr);
	if (Value != NULL)
		return *Value;

	const FGenCompute *GenCompute = FncMap.Find(KeyStr);
	if (!GenCompute)
	    // special exception might be cleaner, but this is internal error, so
	    // it's not that important to localize the text (and exceptions don't
            // work anyway)
	{   CMemPtr<char> Err(100 + KeyStr.GetLength());
	    sprintf((char *)Err, "undefined key %s", Key);
	    throw CGeneralException((char *)Err);
        }

	const FCompute Compute = reinterpret_cast<FCompute>(*GenCompute);
	return (*Compute)(this, Key);
}

template<class CValue>
short COptions<CValue>::Override(const char *Section,
	const char *Key, 
	const CValue &Value)
{
	DString KeyStr = GetKey(Section, Key);

	const CValue *OldValuePtr = ValueMap.Find(KeyStr);
	if (!OldValuePtr)
		return 1;

	ValueMap.Del(KeyStr);
	ValueMap.Add(KeyStr, Value);
	return 0;
}

template<class CValue>
CValue COptions<CValue>::GetMostSpecific(const DString &Section, 
    const char *Key) const
{
    if (Section.IsEmpty())
	return Get(Key);

    DString Compo = GetLongOptionKey(Section, Key);
    if (ValueMap.Find(Compo) != NULL || FncMap.Find(Compo) != NULL)
	return Get(Compo.c_str());
    else
	return Get(Key);
}

template<class CValue>
CValue COptions<CValue>::GetUno(const DString &Name, const char *Key) const
{
     if (Name.IsEmpty())
	     return Get(Key);

	DString Compo = GetLongOptionKey(Name, Key);
	return Get(Compo.c_str());
}

template<class CValue>
void COptions<CValue>::Add(const char *Section, 
    const char *Key, 
    const CValue &Value)
{
    if (!Section)
	Add(Key, Value);
    else
    {   DString LongKey = GetLongOptionKey(DString(Section), Key);
	Add(LongKey.c_str(), Get(Key));
    }
}

template<class CValue>
void COptions<CValue>::Add(const char *Section, const char *Key,
    CValue (*Compute)(const COptions<CValue> *, const char *))
{
    if (!Section)
	Add(Key, Compute);
    else
    {   DString LongKey = GetLongOptionKey(DString(Section), Key);  
	Add(LongKey.c_str(), Get(Key));
    }
}

template<class CValue>
void COptions<CValue>::SetUnoClass(const CStringListValue &List,
     const char *Attr[],
     int AttrCount)
{
    CStringListValue::DIterator Itr = const_cast<CStringListValue &>(
            List).InitIterator();
    while (const_cast<CStringListValue &>(List).IsDereferenceable(Itr))
    {   for (int i = 0; i < AttrCount; i++)
	{   DString Key = GetLongOptionKey(*Itr, Attr[i]);   
	    Add(Key.c_str(), Get(Attr[i]));
	}
	++Itr;
    }
}

#endif
